'''
ypya.base
'''
import json
import logging
import traceback


class YBase(object):
    '''
    basic declaration
    '''

    INCLUDE_DEBUG_FIELD = True
    INCLUDE_EXTRA_FIELD = True
    INCLUDE_FULLY_QUALIFIED_NAME = False
    INCLUDE_LEVEL_SYMBOLIC_NAME = False

    def makePickle(self, record):
        '''
        public method

        :param record: (logging.LogRecord)
        :return: (stringified JSON)
        '''
        obj = GELFData(record)
        setattr(obj, 'parent', self)
        return obj.data


class GELFData(object):
    '''
    data type adapter for GELF
    '''
    SYSLOG_LEVELS = {
        logging.CRITICAL: 2,
        logging.ERROR: 3,
        logging.WARNING: 4,
        logging.INFO: 6,
        logging.DEBUG: 7,
    }

    SPECIAL_NAME_PREFIX = '_'
    GELF_VERSION_NUMBER = '1.0'

    BASIC_FIELDS = (
        'version',
        'timestamp',
        'host',
        'facility',
        'level',
        'short_message',
        'full_message',
    )

    def __init__(self, record):
        '''
        :param record: (logging.LogRecord)
        '''
        assert record is not None, 'invalid argument value'
        self.instance = record

    @property
    def data(self):
        '''
        :return: (string/unicode)
        '''
        self.cache = dict()

        self.collect_basic_fields()

        # rval;
        rval = json.dumps(self.cache)
        return rval

    def collect_basic_fields(self):
        '''
        :return: (dict)
        '''
        self.cache['version'] = self.GELF_VERSION_NUMBER
        self.cache['timestamp'] = self.instance.created
        self.cache['level'] = self.SYSLOG_LEVELS.get(
            self.instance.levelno,
            self.instance.levelno
        )
        self.cache['host'] = self.parent.sender_host
        self.cache['facility'] = self.parent.sender_name or self.instance.name

        formatter = getattr(self, 'formatter', None)
        self.cache['short_message'] = formatter.format(self.instance) if formatter else self.instance.getMessage(),
        self.cache['full_message'] = formatter.format(self.instance) if formatter else self.extract_full_message(self.instance)

        return self.cache

    def extract_full_message(self, record):
        '''
        :param record: (logging.LogRecord)
        '''
        if record.exc_info:
            items = traceback.format_exception(*record.exc_info)
            msg = '\n'.join(items)
        elif record.exc_text:
            msg = record.exc_text
        else:
            msg = record.getMessage()
        return msg



'''
ypya.transport
'''
try:
    from amqplib.client_0_8 import Connection, Message
    BACKEND_NAME = 'amqplib'
except ImportError:
    from amqp import Connection, Message
    BACKEND_NAME = 'amqp'


class AMQPTransport(object):
    '''
    A socket-like interface
    '''

    DEFAULT_CREDENTIAL = ('guest', 'guest')
    DEFAULT_HOSTNAME = 'localhost'
    DEFAULT_PORT = 5672
    DEFAULT_VHOST = '/'
    DEFAULT_EXCHANGE_NAME = 'gelf.exchange'
    DEFAULT_EXCHANGE_TYPE = 'fanout'
    DEFAULT_ROUTING_KEY = ''

    DEFAULT_CONNECTION_TIMEOUT = 1

    DELIVERY_MODE_TRANSIENT  = 1
    DELIVERY_MODE_PERSISTENT = 2

    def __init__(self):
        super(AMQPTransport, self).__init__()
        self.timeout = self.DEFAULT_CONNECTION_TIMEOUT
        self.delivery_mode = self.DELIVERY_MODE_PERSISTENT
        self.backend_name = BACKEND_NAME

    def getSocket(self, parent, conn_args):
        '''
        :param parent: (object)
        :param conn_args: (dict)
        '''
        self.exchange_name = parent.exchange_name
        self.routing_key = parent.routing_key
        is_durable_exchange = getattr(
            parent,
            'exchange_durable',
            True
        )
        allow_auto_deletion = getattr(
            parent,
            'exchange_auto_delete',
            not(is_durable_exchange)
        )

        self._conn = Connection(
            connection_timeout=self.timeout,
            **conn_args,
        )
        self._conn.connect()
        self._channel = self._conn.channel()
        self._channel.exchange_declare(
            exchange=self.exchange_name,
            type=parent.exchange_type,
            durable=is_durable_exchange,
            auto_delete=allow_auto_deletion,
        )
        return self

    def sendall(self, data):
        '''
        public method

        :param data: (obj)
        '''
        pack = Message(
            data,
            delivery_mode=self.delivery_mode,
        )
        self._channel.basic_publish(
            pack,
            exchange=self.exchange_name,
            routing_key=self.routing_key,
        )
        return None

    def close(self):
        '''
        public method
        '''
        try:
            self._conn.close()
        except Exception:
            pass



# -*- python -*-
#
# Copyright 2018, 2019, 2020 Xingeng Chen
# Copyright 2016, 2017 Liang Chen
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
from setuptools import find_packages, setup

class SetupConfig(object):
    CONFIG_HUB = {
        2: {
            'install_requires': ['amqplib',],
        },
        3: {
            'install_requires': ['amqp',],
        },
    }
    def getConfig(self, py_ver=3):
        return self.CONFIG_HUB[py_ver]

setup_args = SetupConfig().getConfig(sys.version_info.major)
setup(**setup_args)
